@@ -2,22 +2,25 @@ import collections
 import numpy as np
 import sys
 import itertools
-from typing import Any, Dict, Iterable, List, Optional, Set, Union
+from typing import Dict, List, Set, Union
 
+from ray.util import log_once
 from ray.rllib.utils.annotations import PublicAPI, DeveloperAPI
 from ray.rllib.utils.compression import pack, unpack, is_compressed
+from ray.rllib.utils.deprecation import deprecation_warning
+from ray.rllib.utils.framework import try_import_tf, try_import_torch
 from ray.rllib.utils.memory import concat_aligned
-from ray.rllib.utils.typing import TensorType
+from ray.rllib.utils.typing import PolicyID, TensorType
+
+tf1, tf, tfv = try_import_tf()
+torch, _ = try_import_torch()
 
 # Default policy id for single agent environments
 DEFAULT_POLICY_ID = "default_policy"
 
-# TODO(ekl) reuse the other id def once we fix imports
-PolicyID = Any
-
 
 @PublicAPI
-class SampleBatch:
+class SampleBatch(dict):
     """Wrapper around a dictionary with string keys and array-like values.
 
     For example, {"obs": [1, 2, 3], "reward": [0, -1, 1]} is a batch of three
@@ -61,30 +64,63 @@ class SampleBatch:
         # Possible seq_lens (TxB or BxT) setup.
         self.time_major = kwargs.pop("_time_major", None)
         self.seq_lens = kwargs.pop("_seq_lens", None)
-        self.max_seq_len = None
-        if self.seq_lens is not None and len(self.seq_lens) > 0:
+        if isinstance(self.seq_lens, list):
+            self.seq_lens = np.array(self.seq_lens)
+        self.dont_check_lens = kwargs.pop("_dont_check_lens", False)
+        self.max_seq_len = kwargs.pop("_max_seq_len", None)
+        if self.max_seq_len is None and self.seq_lens is not None and \
+                not (tf and tf.is_tensor(self.seq_lens)) and \
+                len(self.seq_lens) > 0:
             self.max_seq_len = max(self.seq_lens)
+        self.zero_padded = kwargs.pop("_zero_padded", False)
+        self.is_training = kwargs.pop("_is_training", None)
 
-        # The actual data, accessible by column name (str).
-        self.data = dict(*args, **kwargs)
+        # Call super constructor. This will make the actual data accessible
+        # by column name (str) via e.g. self["some-col"].
+        dict.__init__(self, *args, **kwargs)
+
+        if self.is_training is None:
+            self.is_training = self.pop("is_training", False)
+        if self.seq_lens is None:
+            self.seq_lens = self.get("seq_lens", None)
+
+        self.accessed_keys = set()
+        self.added_keys = set()
+        self.deleted_keys = set()
+        self.intercepted_values = {}
+
+        self.get_interceptor = None
 
         lengths = []
-        for k, v in self.data.copy().items():
+        copy_ = {k: v for k, v in self.items()}
+        for k, v in copy_.items():
             assert isinstance(k, str), self
-            lengths.append(len(v))
+            len_ = len(v) if isinstance(
+                v,
+                (list, np.ndarray)) or (torch and torch.is_tensor(v)) else None
+            lengths.append(len_)
             if isinstance(v, list):
-                self.data[k] = np.array(v)
+                self[k] = np.array(v)
+
         if not lengths:
             raise ValueError("Empty sample batch")
-        assert len(set(lengths)) == 1, \
-            "Data columns must be same length, but lens are {}".format(lengths)
-        if self.seq_lens is not None and len(self.seq_lens) > 0:
+
+        if not self.dont_check_lens:
+            assert len(set(lengths)) == 1, \
+                "Data columns must be same length, but lens are " \
+                "{}".format(lengths)
+
+        if self.seq_lens is not None and \
+                not (tf and tf.is_tensor(self.seq_lens)) and \
+                len(self.seq_lens) > 0:
             self.count = sum(self.seq_lens)
         else:
-            self.count = len(next(iter(self.data.values())))
+            self.count = lengths[0]
 
-        # Keeps track of new columns added after initial ones.
-        self.new_columns = []
+    @PublicAPI
+    def __len__(self):
+        """Returns the amount of samples in the sample batch."""
+        return self.count
 
     @staticmethod
     @PublicAPI
@@ -103,8 +139,13 @@ class SampleBatch:
             return MultiAgentBatch.concat_samples(samples)
         seq_lens = []
         concat_samples = []
+        zero_padded = samples[0].zero_padded
+        max_seq_len = samples[0].max_seq_len
         for s in samples:
             if s.count > 0:
+                assert s.zero_padded == zero_padded
+                if zero_padded:
+                    assert s.max_seq_len == max_seq_len
                 concat_samples.append(s)
                 if s.seq_lens is not None:
                     seq_lens.extend(s.seq_lens)
@@ -117,7 +158,11 @@ class SampleBatch:
         return SampleBatch(
             out,
             _seq_lens=np.array(seq_lens, dtype=np.int32),
-            _time_major=concat_samples[0].time_major)
+            _time_major=concat_samples[0].time_major,
+            _dont_check_lens=True,
+            _zero_padded=zero_padded,
+            _max_seq_len=max_seq_len,
+        )
 
     @PublicAPI
     def concat(self, other: "SampleBatch") -> "SampleBatch":
@@ -148,16 +193,25 @@ class SampleBatch:
         return SampleBatch(out)
 
     @PublicAPI
-    def copy(self) -> "SampleBatch":
+    def copy(self, shallow: bool = False) -> "SampleBatch":
         """Creates a (deep) copy of this SampleBatch and returns it.
 
+        Args:
+            shallow (bool): Whether the copying should be done shallowly.
+
         Returns:
             SampleBatch: A (deep) copy of this SampleBatch object.
         """
-        return SampleBatch(
-            {k: np.array(v, copy=True)
-             for (k, v) in self.data.items()},
-            _seq_lens=self.seq_lens)
+        copy_ = SampleBatch(
+            {
+                k: np.array(v, copy=not shallow)
+                if isinstance(v, np.ndarray) else v
+                for (k, v) in self.items()
+            },
+            _seq_lens=self.seq_lens,
+            _dont_check_lens=self.dont_check_lens)
+        copy_.set_get_interceptor(self.get_interceptor)
+        return copy_
 
     @PublicAPI
     def rows(self) -> Dict[str, TensorType]:
@@ -220,18 +274,26 @@ class SampleBatch:
             List[SampleBatch]: List of batches, one per distinct episode.
         """
 
+        # No eps_id in data -> Make sure there are no "dones" in the middle
+        # and add eps_id automatically.
+        if SampleBatch.EPS_ID not in self:
+            if SampleBatch.DONES in self:
+                assert not any(self[SampleBatch.DONES][:-1])
+            self[SampleBatch.EPS_ID] = np.repeat(0, self.count)
+            return [self]
+
         slices = []
-        cur_eps_id = self.data["eps_id"][0]
+        cur_eps_id = self[SampleBatch.EPS_ID][0]
         offset = 0
         for i in range(self.count):
-            next_eps_id = self.data["eps_id"][i]
+            next_eps_id = self[SampleBatch.EPS_ID][i]
             if next_eps_id != cur_eps_id:
                 slices.append(self.slice(offset, i))
                 offset = i
                 cur_eps_id = next_eps_id
         slices.append(self.slice(offset, self.count))
         for s in slices:
-            slen = len(set(s["eps_id"]))
+            slen = len(set(s[SampleBatch.EPS_ID]))
             assert slen == 1, (s, slen)
         assert sum(s.count for s in slices) == self.count, (slices, self.count)
         return slices
@@ -241,23 +303,59 @@ class SampleBatch:
         """Returns a slice of the row data of this batch (w/o copying).
 
         Args:
-            start (int): Starting index.
+            start (int): Starting index. If < 0, will zero-pad.
             end (int): Ending index.
 
         Returns:
             SampleBatch: A new SampleBatch, which has a slice of this batch's
                 data.
         """
-        if self.time_major is not None:
+        if self.seq_lens is not None and len(self.seq_lens) > 0:
+            if start < 0:
+                data = {
+                    k: np.concatenate([
+                        np.zeros(
+                            shape=(-start, ) + v.shape[1:], dtype=v.dtype),
+                        v[0:end]
+                    ])
+                    for k, v in self.items()
+                }
+            else:
+                data = {k: v[start:end] for k, v in self.items()}
+            # Fix state_in_x data.
+            count = 0
+            state_start = None
+            seq_lens = None
+            for i, seq_len in enumerate(self.seq_lens):
+                count += seq_len
+                if count >= end:
+                    state_idx = 0
+                    state_key = "state_in_{}".format(state_idx)
+                    if state_start is None:
+                        state_start = i
+                    while state_key in self:
+                        data[state_key] = self[state_key][state_start:i + 1]
+                        state_idx += 1
+                        state_key = "state_in_{}".format(state_idx)
+                    seq_lens = list(self.seq_lens[state_start:i]) + [
+                        seq_len - (count - end)
+                    ]
+                    if start < 0:
+                        seq_lens[0] += -start
+                    assert sum(seq_lens) == (end - start)
+                    break
+                elif state_start is None and count > start:
+                    state_start = i
+
             return SampleBatch(
-                {k: v[:, start:end]
-                 for k, v in self.data.items()},
-                _seq_lens=self.seq_lens[start:end],
-                _time_major=self.time_major)
+                data,
+                _seq_lens=np.array(seq_lens, dtype=np.int32),
+                _time_major=self.time_major,
+                _dont_check_lens=True)
         else:
             return SampleBatch(
                 {k: v[start:end]
-                 for k, v in self.data.items()},
+                 for k, v in self.items()},
                 _seq_lens=None,
                 _time_major=self.time_major)
 
@@ -274,41 +372,53 @@ class SampleBatch:
             List[SampleBatch]: The list of (new) SampleBatches (each one of
                 size k).
         """
-        out = []
-        i = 0
-        while i < self.count:
-            out.append(self.slice(i, i + k))
-            i += k
-        return out
+        slices = self._get_slice_indices(k)
+        timeslices = [self.slice(i, j) for i, j in slices]
+        return timeslices
 
-    @PublicAPI
-    def keys(self) -> Iterable[str]:
-        """
-        Returns:
-            Iterable[str]: The keys() iterable over `self.data`.
-        """
-        return self.data.keys()
+    def zero_pad(self, max_seq_len: int, exclude_states: bool = True):
+        """Left zero-pad the data in this SampleBatch in place.
 
-    @PublicAPI
-    def items(self) -> Iterable[TensorType]:
-        """
-        Returns:
-            Iterable[TensorType]: The values() iterable over `self.data`.
-        """
-        return self.data.items()
-
-    @PublicAPI
-    def get(self, key: str) -> Optional[TensorType]:
-        """Returns one column (by key) from the data or None if key not found.
+        This will set the `self.zero_padded` flag to True and
+        `self.max_seq_len` to the given `max_seq_len` value.
 
         Args:
-            key (str): The key (column name) to return.
-
-        Returns:
-            Optional[TensorType]: The data under the given key. None if key
-                not found in data.
+            max_len (int): The max (total) length to zero pad to.
+            exclude_states (bool): If False, also zero-pad all `state_in_x`
+                data. If False, leave `state_in_x` keys as-is.
         """
-        return self.data.get(key)
+        for col in self.keys():
+            # Skip state in columns.
+            if exclude_states is True and col.startswith("state_in_"):
+                continue
+
+            f = self[col]
+            # Save unnecessary copy.
+            if not isinstance(f, np.ndarray):
+                f = np.array(f)
+            # Already good length, can skip.
+            if f.shape[0] == max_seq_len:
+                continue
+            # Generate zero-filled primer of len=max_seq_len.
+            length = len(self.seq_lens) * max_seq_len
+            if f.dtype == np.object or f.dtype.type is np.str_:
+                f_pad = [None] * length
+            else:
+                # Make sure type doesn't change.
+                f_pad = np.zeros((length, ) + np.shape(f)[1:], dtype=f.dtype)
+            # Fill primer with data.
+            f_pad_base = f_base = 0
+            for len_ in self.seq_lens:
+                f_pad[f_pad_base:f_pad_base + len_] = f[f_base:f_base + len_]
+                f_pad_base += max_seq_len
+                f_base += len_
+            assert f_base == len(f), f
+            # Update our data.
+            self[col] = f_pad
+
+        # Set flags to indicate, we are now zero-padded (and to what extend).
+        self.zero_padded = True
+        self.max_seq_len = max_seq_len
 
     @PublicAPI
     def size_bytes(self) -> int:
@@ -316,7 +426,7 @@ class SampleBatch:
         Returns:
             int: The overall size in bytes of the data buffer (all columns).
         """
-        return sum(sys.getsizeof(d) for d in self.data.values())
+        return sum(sys.getsizeof(d) for d in self.values())
 
     @PublicAPI
     def __getitem__(self, key: str) -> TensorType:
@@ -328,7 +438,22 @@ class SampleBatch:
         Returns:
             TensorType: The data under the given key.
         """
-        return self.data[key]
+        # Backward compatibility for when "input-dicts" were used.
+        if key == "is_training":
+            if log_once("SampleBatch['is_training']"):
+                deprecation_warning(
+                    old="SampleBatch['is_training']",
+                    new="SampleBatch.is_training",
+                    error=False)
+            return self.is_training
+
+        self.accessed_keys.add(key)
+        value = dict.__getitem__(self, key)
+        if self.get_interceptor is not None:
+            if key not in self.intercepted_values:
+                self.intercepted_values[key] = self.get_interceptor(value)
+            value = self.intercepted_values[key]
+        return value
 
     @PublicAPI
     def __setitem__(self, key, item) -> None:
@@ -338,9 +463,23 @@ class SampleBatch:
             key (str): The column name to set a value for.
             item (TensorType): The data to insert.
         """
-        if key not in self.data:
-            self.new_columns.append(key)
-        self.data[key] = item
+        # Defend against creating SampleBatch via pickle (no property
+        # `added_keys` and first item is already set).
+        if not hasattr(self, "added_keys"):
+            dict.__setitem__(self, key, item)
+            return
+
+        if key not in self:
+            self.added_keys.add(key)
+
+        dict.__setitem__(self, key, item)
+        if key in self.intercepted_values:
+            self.intercepted_values[key] = item
+
+    @PublicAPI
+    def __delitem__(self, key):
+        self.deleted_keys.add(key)
+        dict.__delitem__(self, key)
 
     @DeveloperAPI
     def compress(self,
@@ -356,12 +495,11 @@ class SampleBatch:
                 compress the obs and new_obs columns.
         """
         for key in columns:
-            if key in self.data:
+            if key in self.keys():
                 if bulk:
-                    self.data[key] = pack(self.data[key])
+                    self[key] = pack(self[key])
                 else:
-                    self.data[key] = np.array(
-                        [pack(o) for o in self.data[key]])
+                    self[key] = np.array([pack(o) for o in self[key]])
 
     @DeveloperAPI
     def decompress_if_needed(self,
@@ -377,26 +515,54 @@ class SampleBatch:
             SampleBatch: This very SampleBatch.
         """
         for key in columns:
-            if key in self.data:
-                arr = self.data[key]
+            if key in self.keys():
+                arr = self[key]
                 if is_compressed(arr):
-                    self.data[key] = unpack(arr)
+                    self[key] = unpack(arr)
                 elif len(arr) > 0 and is_compressed(arr[0]):
-                    self.data[key] = np.array(
-                        [unpack(o) for o in self.data[key]])
+                    self[key] = np.array([unpack(o) for o in self[key]])
         return self
 
-    def __str__(self):
-        return "SampleBatch({})".format(str(self.data))
+    @DeveloperAPI
+    def set_get_interceptor(self, fn):
+        self.get_interceptor = fn
 
     def __repr__(self):
-        return "SampleBatch({})".format(str(self.data))
+        return "SampleBatch({})".format(list(self.keys()))
 
-    def __iter__(self):
-        return self.data.__iter__()
+    def _get_slice_indices(self, slice_size):
+        i = 0
+        slices = []
+        if self.seq_lens is not None and len(self.seq_lens) > 0:
+            start_pos = 0
+            current_slize_size = 0
+            idx = 0
+            while idx < len(self.seq_lens):
+                seq_len = self.seq_lens[idx]
+                current_slize_size += seq_len
+                # Complete minibatch -> Append to slices.
+                if current_slize_size >= slice_size:
+                    slices.append((start_pos, start_pos + slice_size))
+                    start_pos += slice_size
+                    if current_slize_size > slice_size:
+                        overhead = current_slize_size - slice_size
+                        start_pos -= (seq_len - overhead)
+                        idx -= 1
+                    current_slize_size = 0
+                idx += 1
+        else:
+            while i < self.count:
+                slices.append((i, i + slice_size))
+                i += slice_size
+        return slices
 
-    def __contains__(self, x):
-        return x in self.data
+    # TODO: deprecate
+    @property
+    def data(self):
+        if log_once("SampleBatch.data"):
+            deprecation_warning(
+                old="SampleBatch.data[..]", new="SampleBatch[..]", error=False)
+        return self
 
 
 @PublicAPI
@@ -417,16 +583,17 @@ class MultiAgentBatch:
         Args:
             policy_batches (Dict[PolicyID, SampleBatch]): Mapping from policy
                 ids to SampleBatches of experiences.
-            env_steps (int): The number of timesteps in the environment this
-                batch contains. This will be less than the number of
+            env_steps (int): The number of environment steps in the environment
+                this batch contains. This will be less than the number of
                 transitions this batch contains across all policies in total.
         """
 
         for v in policy_batches.values():
             assert isinstance(v, SampleBatch)
         self.policy_batches = policy_batches
-        # Called count for uniformity with SampleBatch. Prefer to access this
-        # via the env_steps() method when possible for clarity.
+        # Called "count" for uniformity with SampleBatch.
+        # Prefer to access this via the `env_steps()` method when possible
+        # for clarity.
         self.count = env_steps
 
     @PublicAPI
@@ -526,7 +693,8 @@ class MultiAgentBatch:
         """
         if len(policy_batches) == 1 and DEFAULT_POLICY_ID in policy_batches:
             return policy_batches[DEFAULT_POLICY_ID]
-        return MultiAgentBatch(policy_batches, env_steps)
+        return MultiAgentBatch(
+            policy_batches=policy_batches, env_steps=env_steps)
 
     @staticmethod
     @PublicAPI
